import logging
import pkg_resources

LOGGER = logging.getLogger(__name__)
PLUGINS = {}
def load():
    for entry in pkg_resources.iter_entry_points(group='mothermayi'):
        runner = entry.load()
        plugin = runner()
        LOGGER.debug("Loaded plugin %s", plugin['name'])
        if plugin['name'] in PLUGINS:
            raise Exception("Already have a plugin with the name {}, cannot overwrite".format(plugin['name']))

        PLUGINS[plugin['name']] = plugin

def get_plugins(name):
    return {k: v for k, v in PLUGINS.items() if name in v}



import os
import yaml

class ConfigNotFoundError(Exception):
    pass

def find_config_file():
    location = os.path.abspath('.')
    while location != '/':
        config = os.path.join(location, '.mothermayi')
        if os.path.exists(config):
            return config
        location = os.path.dirname(location)
    raise ConfigNotFoundError("Could not find .mothermayi")

def parse():
    try:
        configfile = find_config_file()
    except ConfigNotFoundError:
        return {}

    with open(configfile, 'r') as f:
        config = yaml.load(f)
    return config



import logging
import mothermayi.entryway
import mothermayi.errors
import mothermayi.git
import mothermayi.colors

LOGGER = logging.getLogger(__name__)

def handle_plugins(config, plugins, staged):
    successful = True
    for name, plugin in plugins.items():
        try:
            result = plugin['pre-commit'](config, staged)
            status = mothermayi.colors.green("PASSED")
        except mothermayi.errors.FailHook as e:
            result = str(e)
            status = mothermayi.colors.red("FAILED")
            successful = False
        message = "{0:<10}...[{1:<7}]".format(name, status)
        print(message)
        if result:
            print(result)
    return successful

def run(config):
    with mothermayi.git.stash():
        staged = mothermayi.git.get_staged()
        if not staged:
            return
        plugins = mothermayi.entryway.get_plugins('pre-commit')
        if not handle_plugins(config, plugins, staged):
            return 1
        return 0



import contextlib
import logging
import re
import mothermayi.process

LOGGER = logging.getLogger(__name__)

@contextlib.contextmanager
def stash():
    mothermayi.process.execute(['git', 'stash', '-u', '--keep-index'])
    try:
       yield
    finally:
        mothermayi.process.execute(['git', 'reset', '--hard'])
        mothermayi.process.execute(['git', 'stash', 'pop', '--quiet', '--index'])


IS_MODIFIED = re.compile(r'^[MA]\s+(?P<filename>.*)$')
def get_staged():
    staged = []
    stdout, stderr = mothermayi.process.execute(['git', 'status', '--porcelain'])
    output = stdout.decode('utf-8')
    for line in output.splitlines():
        match = IS_MODIFIED.match(line)
        if match:
            staged.append(match.group('filename'))
    return staged



import logging
import subprocess

LOGGER = logging.getLogger(__name__)

def execute(command):
    LOGGER.debug("Executing %s", ' '.join(command))
    proc = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    stdout, stderr = proc.communicate(input=None)
    if proc.returncode != 0:
        raise subprocess.CalledProcessError(proc.returncode, command, "Failed to execute comand")
    LOGGER.debug("stdout %s", stdout)
    LOGGER.debug("stderr %s", stderr)
    return stdout, stderr




BLUE = '\033[94m'
GREEN = '\033[92m'
YELLOW = '\033[93m'
RED = '\033[91m'
ENDC = '\033[0m'
BOLD = '\033[1m'
UNDERLINE = '\033[4m'

def green(text):
    return GREEN + text + ENDC

def red(text):
    return RED + text + ENDC



import contextlib
import os
import re
import subprocess

from setuptools import setup
from setuptools.command.sdist import sdist

DATA_ROOTS = []
PROJECT = 'mothermayi'
VERSION_FILE = 'mothermayi/__init__.py'

def _get_output_or_none(args):
    try:
        return subprocess.check_output(args).decode('utf-8').strip()
    except subprocess.CalledProcessError:
        return None

def _get_git_description():
    return _get_output_or_none(['git', 'describe'])

def _get_git_branches_for_this_commit():
    branches = _get_output_or_none(['git', 'branch', '-r', '--contains', 'HEAD'])
    split = branches.split('\n') if branches else []
    return [branch.strip() for branch in split]

def _is_on_releasable_branch(branches):
    return any(['origin/master' == branch or branch.startswith('origin/hotfix') for branch in branches])

def _git_to_version(git):
    match = re.match(r'(?P<tag>[\d\.]+)-(?P<offset>[\d]+)-(?P<sha>\w{8})', git)
    if not match:
        version = git
    else:
        version = "{tag}.post0.dev{offset}".format(**match.groupdict())
    return version

def _get_version_from_git():
    git_description = _get_git_description()
    git_branches = _get_git_branches_for_this_commit()
    version = _git_to_version(git_description) if git_description else None
    if git_branches and not _is_on_releasable_branch(git_branches):
        print("Forcing version to 0.0.1 because this commit is on branches {} and not a whitelisted branch".format(git_branches))
        version = '0.0.1'
    return version

VERSION_REGEX = re.compile(r'__version__ = "(?P<version>[\w\.]+)"')
def _get_version_from_file():
    with open(VERSION_FILE, 'r') as f:
        content = f.read()
    match = VERSION_REGEX.match(content)
    if not match:
        raise Exception("Failed to pull version out of '{}'".format(content))
    version = match.group(1)
    return version

@contextlib.contextmanager
def write_version():
    version = _get_version_from_git()
    if version:
        with open(VERSION_FILE, 'r') as version_file:
            old_contents = version_file.read()
        with open(VERSION_FILE, 'w') as version_file:
            new_contents = '__version__ = "{}"\n'.format(version)
            version_file.write(new_contents)
    print("Wrote {} with {}".format(VERSION_FILE, new_contents))
    yield
    if version:
        with open(VERSION_FILE, 'w') as version_file:
            version_file.write(old_contents)
            print("Reverted {} to old contents".format(VERSION_FILE))

def get_version():
    file_version = _get_version_from_file()
    git_version = _get_version_from_git()
    return (file_version == 'development' and git_version) or file_version

def get_data_files():
    data_files = []
    for data_root in DATA_ROOTS:
        for root, _, files in os.walk(data_root):
            data_files.append((os.path.join(PROJECT, root), [os.path.join(root, f) for f in files]))
    return data_files

class CustomSDistCommand(sdist): # pylint: disable=no-init
    def run(self):
        with write_version():
            sdist.run(self)

def main():
    setup(
        name                 = "mothermayi",
        version              = get_version(),
        description          = "A system for installing and managing git commit hooks",
        url                  = "https://github.com/EliRibble/mothermayi",
        long_description     = open("README.md").read(),
        author               = "EliRibble",
        author_email         = "junk@theribbles.org",
        cmdclass             = {
            'sdist'         : CustomSDistCommand,
        },
        install_requires    = [
            'PyYAML==3.11',
        ],
        extras_require={
            'develop': [
                'pytest==2.7.3',
            ]
        },
        packages             = ["mothermayi"],
        package_data         = {
            "mothermayi"     : ["mothermayi/*"],
        },
        data_files           = get_data_files(),
        entry_points = {
        },
        scripts = ['bin/mothermayi'],
        include_package_data = True,
    )

if __name__ == "__main__":
    main()



import logging
import os
import stat

LOGGER = logging.getLogger(__name__)

class NoRepoFoundError(Exception):
    pass

class PreCommitExists(Exception):
    pass

def find_git_repo():
    location = os.path.abspath('.')
    while location != '/':
        check = os.path.join(location, '.git')
        if os.path.exists(check) and os.path.isdir(check):
            return check
        location = os.path.dirname(location)
    raise NoRepoFoundError("Could not find a git repository (.git) in {}".format(os.path.abspath('.')))

HOOK_CONTENT = """
mothermayi pre-commit
"""

def write_hook(pre_commit):
    if os.path.exists(pre_commit):
        with open(pre_commit, 'r') as f:
            if f.read() == HOOK_CONTENT:
                LOGGER.debug("Not writing the hook - it's already set")
            else:
                raise PreCommitExists("A git hook already exists at {}. Refusing to overwrite. Please remove it manually".format(pre_commit))
    else:
        with open(pre_commit, 'w') as f:
            f.write(HOOK_CONTENT)
    original = os.stat(pre_commit)
    os.chmod(pre_commit, original.st_mode | stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH)
    LOGGER.debug("Set %s to executable", pre_commit)

def install(config):
    repo = find_git_repo()
    LOGGER.debug("Found git repo at %s", repo)
    hooks = os.path.join(repo, 'hooks')
    pre_commit = os.path.join(hooks, 'pre-commit')
    write_hook(pre_commit)
